use bevy::prelude::*; // Removed unused AppExit
use std::time::Duration;
// --- Game States ---
#[derive(Clone, Eq, PartialEq, Debug, Hash, Default, States)]
enum AppState {
    #[default]
    Playing,
    GameOver,
}
// --- Constants ---
const WINDOW_WIDTH: f32 = 600.0;
const WINDOW_HEIGHT: f32 = 800.0;
const PLAYER_SPEED: f32 = 300.0;
const BULLET_SPEED: f32 = 500.0;
const ENEMY_SPEED: f32 = 100.0;
const PLAYER_SIZE: Vec2 = Vec2::new(30.0, 30.0);
const ENEMY_SIZE: Vec2 = Vec2::new(40.0, 40.0);
const BULLET_SIZE: Vec2 = Vec2::new(5.0, 15.0);

const STARTING_LIVES: u32 = 3;
const PLAYER_RESPAWN_DELAY: f32 = 2.0;
const PLAYER_INVINCIBILITY_DURATION: f32 = 2.0;

// Collision thresholds
const BULLET_ENEMY_COLLISION_THRESHOLD: f32 = (BULLET_SIZE.x + ENEMY_SIZE.x) * 0.5; // 22.5
const PLAYER_ENEMY_COLLISION_THRESHOLD: f32 = (PLAYER_SIZE.x + ENEMY_SIZE.x) * 0.5; // 35.0

// --- Components ---
#[derive(Component)]
struct Player {
    speed: f32,
    shoot_cooldown: Timer,
}

#[derive(Component)]
struct Bullet;

#[derive(Component)]
struct Enemy;

#[derive(Component)]
struct Invincible {
    timer: Timer,
}

// --- Resources ---
#[derive(Resource)]
struct EnemySpawnTimer {
    timer: Timer,
}

#[derive(Resource)]
struct PlayerLives {
    count: u32,
}

#[derive(Resource)]
struct PlayerRespawnTimer {
    timer: Timer,
}

// --- Game Over UI ---

#[derive(Component)]
struct GameOverUI;

fn setup_game_over_ui(mut commands: Commands) {
   println!("Entering GameOver state. Setting up UI.");
   commands.spawn((
       TextBundle::from_section(
           "GAME OVER",
           TextStyle {
               font_size: 100.0,
               color: Color::WHITE,
               ..default()
           },
       )
       .with_style(Style {
           position_type: PositionType::Absolute,
           align_self: AlignSelf::Center,
           justify_self: JustifySelf::Center,
           top: Val::Percent(40.0), // Center vertically roughly
           ..default()
       }),
       GameOverUI, // Tag the UI element
   ));
   // TODO: Add "Press R to Restart" text later
}

fn cleanup_game_over_ui(mut commands: Commands, query: Query<Entity, With<GameOverUI>>) {
    println!("Exiting GameOver state. Cleaning up UI.");
   for entity in query.iter() {
       commands.entity(entity).despawn_recursive();
   }
}

// --- Cleanup ---

fn cleanup_game_entities(
   mut commands: Commands,
   bullet_query: Query<Entity, With<Bullet>>,
   enemy_query: Query<Entity, With<Enemy>>,
   // Optionally despawn player too, or handle separately if needed for restart
   // player_query: Query<Entity, With<Player>>,
) {
   println!("Exiting Playing state. Cleaning up game entities.");
   for entity in bullet_query.iter() {
       commands.entity(entity).despawn();
   }
   for entity in enemy_query.iter() {
       commands.entity(entity).despawn();
   }
   // for entity in player_query.iter() {
   //     commands.entity(entity).despawn();
   // }
}

fn main() {
    App::new() // Start App builder
        .init_state::<AppState>() // Initialize the AppState *after* App::new()
        .insert_resource(ClearColor(Color::rgb(0.0, 0.0, 0.0)))
        .add_plugins(DefaultPlugins.set(WindowPlugin {
            primary_window: Some(Window {
                title: "Galaga :: Lives: 3".into(), // Initial title
                resolution: (WINDOW_WIDTH, WINDOW_HEIGHT).into(),
                ..default()
            }),
            ..default()
        }))
        // Add Resources
        .insert_resource(PlayerLives { count: STARTING_LIVES })
        .insert_resource(PlayerRespawnTimer {
             // Start paused and finished
            timer: Timer::new(Duration::from_secs_f32(PLAYER_RESPAWN_DELAY), TimerMode::Once)
        })
        .insert_resource(EnemySpawnTimer {
            timer: Timer::new(Duration::from_secs_f32(0.5), TimerMode::Repeating),
        })
        // Add Systems
        .add_systems(Startup, setup)
        // Systems running only when Playing
        .add_systems(Update, (
            move_player,
            player_shoot.run_if(player_exists),
            move_bullets,
            move_enemies,
            spawn_enemies,
            check_bullet_collisions,
            check_player_enemy_collisions.run_if(player_vulnerable),
            respawn_player.run_if(should_respawn_player),
            manage_invincibility,
            // Game Over check is now implicit in check_player_enemy_collisions
        ).run_if(in_state(AppState::Playing)))
        // Systems running regardless of state (or managing state transitions)
        .add_systems(Update, (
            update_window_title, // Keep title updated
            // Add system to check for restart input in GameOver state later
            bevy::window::close_on_esc, // Allow closing anytime
        ))
        // Systems for entering/exiting states
        .add_systems(OnEnter(AppState::GameOver), setup_game_over_ui)
        .add_systems(OnExit(AppState::GameOver), cleanup_game_over_ui)
        .add_systems(OnExit(AppState::Playing), cleanup_game_entities) // Cleanup when leaving Playing
        .run();
}

// --- Run Conditions ---
fn player_exists(query: Query<&Player>) -> bool {
    !query.is_empty()
}

fn player_vulnerable(query: Query<&Player, Without<Invincible>>) -> bool {
    !query.is_empty()
}

fn should_respawn_player(
    lives: Res<PlayerLives>,
    player_query: Query<&Player>,
) -> bool {
    player_query.is_empty() && lives.count > 0
}


// --- Systems ---

fn setup(mut commands: Commands) {
    commands.spawn(Camera2dBundle::default());
    spawn_player_ship(&mut commands); // Use helper function
}

// Helper to spawn player (used in setup and respawn)
fn spawn_player_ship(commands: &mut Commands) {
     commands.spawn((
        SpriteBundle {
            sprite: Sprite {
                color: Color::rgb(0.0, 0.5, 1.0),
                custom_size: Some(PLAYER_SIZE),
                ..default()
            },
            transform: Transform::from_translation(Vec3::new(0.0, -WINDOW_HEIGHT / 2.0 + PLAYER_SIZE.y / 2.0 + 20.0, 0.0)),
            ..default()
        },
        Player {
            speed: PLAYER_SPEED,
            shoot_cooldown: Timer::new(Duration::from_secs_f32(0.3), TimerMode::Once),
        },
        // Player starts invincible for a short time
        Invincible {
            timer: Timer::new(Duration::from_secs_f32(PLAYER_INVINCIBILITY_DURATION), TimerMode::Once)
        }
    ));
    println!("Player spawned!");
}


fn move_player(
    keyboard_input: Res<ButtonInput<KeyCode>>,
    mut query: Query<(&mut Transform, &Player)>,
    time: Res<Time>,
) {
    // Using get_single_mut handles the case where player might not exist yet (or was just destroyed)
    if let Ok((mut transform, player)) = query.get_single_mut() {
        let mut direction = 0.0;

        if keyboard_input.pressed(KeyCode::KeyA) || keyboard_input.pressed(KeyCode::ArrowLeft) {
            direction -= 1.0;
        }

        if keyboard_input.pressed(KeyCode::KeyD) || keyboard_input.pressed(KeyCode::ArrowRight) {
            direction += 1.0;
        }

        transform.translation.x += direction * player.speed * time.delta_seconds();
        let half_player_width = PLAYER_SIZE.x / 2.0;
        transform.translation.x = transform.translation.x.clamp(-WINDOW_WIDTH / 2.0 + half_player_width, WINDOW_WIDTH / 2.0 - half_player_width);
    }
}

fn player_shoot(
    keyboard_input: Res<ButtonInput<KeyCode>>,
    mut query: Query<(&Transform, &mut Player)>, // Should only run if player exists due to run_if
    mut commands: Commands,
    time: Res<Time>,
) {
   if let Ok((player_transform, mut player)) = query.get_single_mut() {
        player.shoot_cooldown.tick(time.delta());

        if (keyboard_input.just_pressed(KeyCode::Space) || keyboard_input.just_pressed(KeyCode::ArrowUp))
            && player.shoot_cooldown.finished() {
            player.shoot_cooldown.reset();

            let bullet_start_pos = player_transform.translation + Vec3::Y * (PLAYER_SIZE.y / 2.0 + BULLET_SIZE.y / 2.0);

            commands.spawn((
                SpriteBundle {
                    sprite: Sprite {
                        color: Color::rgb(1.0, 1.0, 1.0),
                        custom_size: Some(BULLET_SIZE),
                        ..default()
                    },
                    transform: Transform::from_translation(bullet_start_pos),
                    ..default()
                },
                Bullet,
            ));
        }
    }
}

fn move_bullets(
    mut query: Query<(Entity, &mut Transform), With<Bullet>>,
    time: Res<Time>,
    mut commands: Commands,
) {
    for (entity, mut transform) in query.iter_mut() {
        transform.translation.y += BULLET_SPEED * time.delta_seconds();

        if transform.translation.y > WINDOW_HEIGHT / 2.0 + BULLET_SIZE.y {
            commands.entity(entity).despawn();
        }
    }
}

fn spawn_enemies(
    mut commands: Commands,
    time: Res<Time>,
    mut timer: ResMut<EnemySpawnTimer>,
) {
    timer.timer.tick(time.delta());

    if timer.timer.just_finished() {
        let x_pos = (fastrand::f32() - 0.5) * (WINDOW_WIDTH - ENEMY_SIZE.x);

        commands.spawn((
            SpriteBundle {
                sprite: Sprite {
                    color: Color::rgb(1.0, 0.2, 0.2),
                    custom_size: Some(ENEMY_SIZE),
                    ..default()
                },
                transform: Transform::from_translation(Vec3::new(
                    x_pos,
                    WINDOW_HEIGHT / 2.0 - ENEMY_SIZE.y / 2.0,
                    0.0
                )),
                ..default()
            },
            Enemy,
        ));
    }
}

fn move_enemies(
    mut query: Query<(Entity, &mut Transform), With<Enemy>>,
    time: Res<Time>,
    mut commands: Commands,
) {
    for (entity, mut transform) in query.iter_mut() {
        transform.translation.y -= ENEMY_SPEED * time.delta_seconds();

        if transform.translation.y < -WINDOW_HEIGHT / 2.0 - ENEMY_SIZE.y {
            commands.entity(entity).despawn();
        }
    }
}

fn check_bullet_collisions(
    mut commands: Commands,
    bullet_query: Query<(Entity, &Transform), With<Bullet>>,
    enemy_query: Query<(Entity, &Transform), With<Enemy>>,
) {
    for (bullet_entity, bullet_transform) in bullet_query.iter() {
        for (enemy_entity, enemy_transform) in enemy_query.iter() {
            let distance = bullet_transform.translation.distance(enemy_transform.translation);

            if distance < BULLET_ENEMY_COLLISION_THRESHOLD {
                commands.entity(bullet_entity).despawn();
                commands.entity(enemy_entity).despawn();
                break;
            }
        }
    }
}

// Modified Collision Check for Player
fn check_player_enemy_collisions(
    mut commands: Commands,
    mut lives: ResMut<PlayerLives>,
    mut respawn_timer: ResMut<PlayerRespawnTimer>,
    mut next_state: ResMut<NextState<AppState>>, // Resource to change state
    // Query player without Invincible component - relies on run_if condition too
    player_query: Query<(Entity, &Transform), (With<Player>, Without<Invincible>)>, 
    enemy_query: Query<(Entity, &Transform), With<Enemy>>,
) {
    // This system only runs if player exists and is not invincible, due to run_if
    if let Ok((player_entity, player_transform)) = player_query.get_single() {
        for (enemy_entity, enemy_transform) in enemy_query.iter() {
            let distance = player_transform.translation.distance(enemy_transform.translation);

            if distance < PLAYER_ENEMY_COLLISION_THRESHOLD {
                println!("Player hit!");
                commands.entity(enemy_entity).despawn(); // Despawn enemy

                lives.count = lives.count.saturating_sub(1); // Decrement lives safely
                println!("Lives remaining: {}", lives.count);

                commands.entity(player_entity).despawn(); // Despawn player

                if lives.count > 0 {
                    // Start the respawn timer
                    respawn_timer.timer.reset();
                    respawn_timer.timer.unpause();
                    println!("Respawn timer started.");
                } else {
                    println!("GAME OVER!");
                    next_state.set(AppState::GameOver); // Transition to GameOver state
                }
                // Important: Break after handling one collision per frame for the player
                break;
            }
        }
    }
}

// New System: Respawn Player
fn respawn_player(
    mut commands: Commands,
    time: Res<Time>,
    mut respawn_timer: ResMut<PlayerRespawnTimer>,
    // No player query needed here due to run_if condition
) {
    // Tick the timer only if it's actually running
    if respawn_timer.timer.tick(time.delta()).just_finished() {
         println!("Respawn timer finished. Spawning player.");
         spawn_player_ship(&mut commands);
         respawn_timer.timer.pause(); // Pause timer until next death
    }
}

// New System: Manage Invincibility
fn manage_invincibility(
    mut commands: Commands,
    time: Res<Time>,
    mut query: Query<(Entity, &mut Invincible, Option<&mut Visibility>), With<Player>>,
) {
    for (entity, mut invincible, mut visibility) in query.iter_mut() {
        invincible.timer.tick(time.delta());

        // Blinking effect (optional)
        if let Some(ref mut vis) = visibility {
             // Blink roughly 5 times per second
            let elapsed_secs = invincible.timer.elapsed_secs();
            **vis = if (elapsed_secs * 10.0).floor() % 2.0 == 0.0 {
                Visibility::Visible
            } else {
                Visibility::Hidden
            };
        } 
        

        if invincible.timer.finished() {
            println!("Invincibility finished.");
            commands.entity(entity).remove::<Invincible>();
             // Ensure player is visible when invincibility ends
             if let Some(ref mut vis) = visibility {
                  **vis = Visibility::Visible;
            }
        }
    }
}

// New System: Update Window Title with Lives
fn update_window_title(
    lives: Res<PlayerLives>,
    mut windows: Query<&mut Window>,
) {
    if lives.is_changed() {
        let mut window = windows.single_mut();
        window.title = format!("Galaga :: Lives: {}", lives.count);
    }
}
